import re

import django
from django.contrib.auth import HASH_SESSION_KEY, get_user_model
from django.contrib.sites.models import Site
from django.core import mail
from django.test import TestCase, override_settings

if django.VERSION >= (1, 10):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

User = get_user_model()


def prevent_logout_pwd_change(client, user):
    """
    Updating a user's password logs out all sessions for the user.
    By calling this function this behavior will be prevented.

    See this link, and the source code of `update_session_auth_hash`:
    https://docs.djangoproject.com/en/dev/topics/auth/default/#session-invalidation-on-password-change
    """
    if hasattr(user, 'get_session_auth_hash'):
        session = client.session
        session[HASH_SESSION_KEY] = user.get_session_auth_hash()
        session.save()


class ViewsTests(TestCase):
    """
    Checks (barely) that templates do not contain errors.
    """
    def setUp(self):
        self.u = User.objects.create_user('user', 'user@mail.net', 'user')

        Site.objects.filter(pk=1).update(domain='testserver')

    def _login(self, client=None):
        if client is None:
            client = self.client
        client.login(username='user', password='user')

    def _get_confirm_email_link(self, email_msg):
        m = re.search(
            r'http://testserver(/accounts/confirm-email/.*/)',
            email_msg.body,
        )
        return m.group(1)

    def _get_reset_password_link(self, email_msg):
        m = re.search(
            r'http://testserver(/accounts/password/reset/key/.*/)',
            email_msg.body,
        )
        return m.group(1)

    def test_account_signup(self):
        r = self.client.get(reverse('account_signup'))
        self.assertEqual(r.status_code, 200)

    @override_settings(
        ACCOUNT_ADAPTER='tests.adapter.ClosedSignupAccountAdapter',
    )
    def test_account_closed_signup(self):
        r = self.client.get(reverse('account_signup'))
        self.assertEqual(r.status_code, 200)

    def test_account_login(self):
        r = self.client.get(reverse('account_login'))
        self.assertEqual(r.status_code, 200)

    def test_account_logout(self):
        self._login()
        r = self.client.get(reverse('account_logout'))
        self.assertEqual(r.status_code, 200)

    def test_account_change_password(self):
        self._login()
        r = self.client.get(reverse('account_change_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_set_password(self):
        self._login()
        self.u.set_unusable_password()
        self.u.save()
        prevent_logout_pwd_change(self.client, self.u)

        r = self.client.get(reverse('account_set_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_inactive(self):
        r = self.client.get(reverse('account_inactive'))
        self.assertEqual(r.status_code, 200)

    def test_account_email(self):
        self._login()
        r = self.client.get(reverse('account_email'))
        self.assertEqual(r.status_code, 200)

    def test_account_email_verification_sent(self):
        self._login()
        r = self.client.get(reverse('account_email_verification_sent'))
        self.assertEqual(r.status_code, 200)

    def test_account_confirm_email(self):
        self._login()
        self.client.post(reverse('account_email'), {
            'action_add': '',
            'email': 'test@mail.net',
        })
        confirm_url = self._get_confirm_email_link(mail.outbox[0])

        r = self.client.get(confirm_url)
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password(self):
        r = self.client.get(reverse('account_reset_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_done(self):
        r = self.client.get(reverse('account_reset_password_done'))
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_from_key(self):
        self.client.post(reverse('account_reset_password'), {
            'email': 'user@mail.net',
        })
        reset_url = self._get_reset_password_link(mail.outbox[0])

        r = self.client.get(reset_url, follow=True)
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_from_key_done(self):
        r = self.client.get(reverse('account_reset_password_from_key_done'))
        self.assertEqual(r.status_code, 200)



# -*- coding: utf-8 -*-
from allauth.account.models import EmailAddress
from allauth.socialaccount.providers.base import ProviderAccount

from allauth_cas.providers import CASProvider


class ClipperAccount(ProviderAccount):
    pass


class ClipperProvider(CASProvider):
    id = 'clipper'
    name = 'Clipper'
    account_class = ClipperAccount

    def extract_email(self, data):
        uid, extra = data
        return '{}@clipper.ens.fr'.format(uid.strip().lower())

    def extract_common_fields(self, data):
        common = super(ClipperProvider, self).extract_common_fields(data)
        common['email'] = self.extract_email(data)
        return common

    def extract_email_addresses(self, data):
        return [
            EmailAddress(
                email=self.extract_email(data),
                verified=True, primary=True,
            ),
        ]

    def extract_extra_data(self, data):
        extra_data = super(ClipperProvider, self).extract_extra_data(data)
        extra_data['email'] = self.extract_email(data)
        return extra_data

    def message_suggest_caslogout_on_logout(self, request):
        return (
            self.get_settings()
            .get('MESSAGE_SUGGEST_CASLOGOUT_ON_LOGOUT', True)
        )


provider_classes = [ClipperProvider]



from django.contrib.auth import get_user_model

from allauth_cas.test.testcases import CASTestCase, CASViewTestCase

User = get_user_model()


class ClipperProviderTests(CASTestCase):

    def setUp(self):
        self.u = User.objects.create_user('user', 'user@mail.net', 'user')

    def test_auto_signup(self):
        self.client_cas_login(
            self.client, provider_id='clipper', username='clipper_uid')

        u = User.objects.get(username='clipper_uid')
        self.assertEqual(u.email, 'clipper_uid@clipper.ens.fr')


class ClipperViewsTests(CASViewTestCase):

    def test_login_view(self):
        r = self.client.get('/accounts/clipper/login/')
        expected = (
            "https://cas.eleves.ens.fr/login?service=http%3A%2F%2Ftestserver"
            "%2Faccounts%2Fclipper%2Flogin%2Fcallback%2F"
        )
        self.assertRedirects(
            r, expected,
            fetch_redirect_response=False,
        )

    def test_callback_view(self):
        # Required to initialize a SocialLogin.
        r = self.client.get('/accounts/clipper/login/')

        # Tests.
        self.patch_cas_response(valid_ticket='__all__')
        r = self.client.get('/accounts/clipper/login/callback/', {
            'ticket': '123456',
        })
        self.assertLoginSuccess(r)

    def test_logout_view(self):
        r = self.client.get('/accounts/clipper/logout/')
        expected = (
            "https://cas.eleves.ens.fr/logout?service=http%3A%2F%2Ftestserver"
            "%2F"
        )
        self.assertRedirects(
            r, expected,
            fetch_redirect_response=False,
        )



import django
from django import template
from django.conf import settings
from django.shortcuts import resolve_url

from allauth.account.adapter import get_adapter

register = template.Library()

if django.VERSION >= (1, 9):
    simple_tag = register.simple_tag
else:
    simple_tag = register.assignment_tag


@simple_tag
def get_home_url():
    home_url = getattr(settings, 'ACCOUNT_HOME_URL', None)
    if home_url is None:
        return None
    return resolve_url(home_url)


@simple_tag
def get_profile_url():
    profile_url = getattr(settings, 'ACCOUNT_DETAILS_URL', None)
    if profile_url is None:
        return None
    return resolve_url(profile_url)


@simple_tag(takes_context=True)
def is_open_for_signup(context):
    request = context['request']
    return get_adapter(request).is_open_for_signup(request)



# -*- coding: utf-8 -*-
from allauth_cas import views

from .provider import ClipperProvider


class ClipperCASAdapter(views.CASAdapter):
    provider_id = ClipperProvider.id
    url = 'https://cas.eleves.ens.fr'
    version = 3


login = views.CASLoginView.adapter_view(ClipperCASAdapter)
callback = views.CASCallbackView.adapter_view(ClipperCASAdapter)
logout = views.CASLogoutView.adapter_view(ClipperCASAdapter)



import os

from setuptools import find_packages, setup

from allauth_ens import __version__

BASE_DIR = os.path.dirname(__file__)

with open(os.path.join(BASE_DIR, 'README.rst')) as readme:
    README = readme.read()

setup(
    name='django-allauth-ens',
    version=__version__,
    description=(
        "Providers for django-allauth allowing using the ENS' auth-systems."
    ),
    author='cof-geek',
    author_email='cof-geek@ens.fr',
    keywords='django allauth cas authentication',
    long_description=README,
    url='https://git.eleves.ens.fr/cof-geek/django-allauth-ens',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ],
    license='MIT',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    install_requires=[
        'django-allauth',
        'django-allauth-cas>=1.0.0b2,<1.1',
        'django-widget-tweaks',
    ],
)
